from celery import shared_task

from django.utils.module_loading import import_string

from .app_settings import app_settings


@shared_task
def send(*args, **kwargs):
    backend = import_string(app_settings.BACKEND_FOR_QUEUE)()

    return backend.send(*args, **kwargs)



from django.conf import settings


def setting(suffix, default):
    @property
    def fn(self):
        return getattr(settings, 'SLACK_{}'.format(suffix), default)

    return fn


class AppSettings(object):
    DEFAULT_ENDPOINT_URL = 'https://slack.com/api/chat.postMessage'
    DEFAULT_BACKEND = (
        'django_slack.backends.DisabledBackend'
        if settings.DEBUG
        else 'django_slack.backends.UrllibBackend'
    )

    TOKEN = setting('TOKEN', None)
    CHANNEL = setting('CHANNEL', '#general')
    USERNAME = setting('USERNAME', 'bot')
    ICON_URL = setting('ICON_URL', None)
    ICON_EMOJI = setting('ICON_EMOJI', None)

    ENDPOINT_URL = setting('ENDPOINT_URL', DEFAULT_ENDPOINT_URL)

    BACKEND = setting('BACKEND', DEFAULT_BACKEND)
    BACKEND_FOR_QUEUE = setting('BACKEND_FOR_QUEUE', DEFAULT_BACKEND)

    FAIL_SILENTLY = setting('FAIL_SILENTLY', False)
    AS_USER = setting('AS_USER', False)


app_settings = AppSettings()



import django
import unittest

from django_slack import slack_message
from django_slack.utils import get_backend


class SlackTestCase(unittest.TestCase):
    def setUp(self):
        self.backend = get_backend()
        self.backend.reset()

    def assertMessageCount(self, count):
        self.assertEqual(len(self.backend.messages), count)

    def assertMessage(self, url=None, **kwargs):
        """
        Ensure there was only one message sent with a URL and data values.
        """

        self.assertMessageCount(1)
        message = self.backend.messages[0]

        # Optionally ensure the URL.
        if url is not None:
            self.assertEqual(url, message['url'])

        # Ensure each input value in data.
        for kwarg, value in kwargs.items():
            self.assertEqual(value, message['message_data'][kwarg])


class TestEscaping(SlackTestCase):
    def test_simple_message(self):
        slack_message('test.slack', {'text': 'test'})
        self.assertMessage(text='test')

    def test_escaped(self):
        """
        Simple test of the Django escaping to illustrate problem.
        """
        slack_message('test.slack', {'text': '< > & " \''})
        if django.VERSION[0] >= 3:
            self.assertMessage(text='&lt; &gt; &amp; &quot; &#x27;')
        else:
            self.assertMessage(text='&lt; &gt; &amp; &quot; &#39;')

    def test_escape_tag(self):
        """
        Test using the escape tag, but not escaping anything.
        """
        slack_message('escape.slack', {'text': 'test'})
        self.assertMessage(text='test')

    def test_escape_chars(self):
        """
        Test the characters Slack wants escaped.

        See <https://api.slack.com/docs/formatting#how_to_escape_characters>
        """
        slack_message('escape.slack', {'text': '< > &'})
        self.assertMessage(text='&lt; &gt; &amp;')

    def test_not_escape_chars(self):
        """
        Test normal HTML escaped characters that Slack doesn't want escaped.
        """
        slack_message('escape.slack', {'text': '" \''})
        self.assertMessage(text='" \'')



from django import template
from django.utils.encoding import force_str
from django.utils.functional import keep_lazy
from django.utils.safestring import SafeText, mark_safe
from django.template.defaultfilters import stringfilter

register = template.Library()

_slack_escapes = {
    ord('&'): u'&amp;',
    ord('<'): u'&lt;',
    ord('>'): u'&gt;',
}


@keep_lazy(str, SafeText)
@register.filter(is_safe=True)
@stringfilter
def escapeslack(value):
    """
    Returns the given text with ampersands and angle brackets encoded for use in
    the Slack API, per the Slack API documentation:
    <https://api.slack.com/docs/formatting#how_to_escape_characters>

    This is based on django.template.defaultfilters.escapejs.
    """
    return mark_safe(force_str(value).translate(_slack_escapes))



import pprint
import logging
import urllib.request

from django.http.request import QueryDict
from django.utils.module_loading import import_string

from .utils import Backend
from .app_settings import app_settings

logger = logging.getLogger(__name__)


class UrllibBackend(Backend):
    def send(self, url, message_data, **kwargs):
        qs = QueryDict(mutable=True)
        qs.update(message_data)

        r = urllib.request.urlopen(
            urllib.request.Request(url, qs.urlencode().encode('utf-8'),)
        )

        result = r.read().decode('utf-8')

        return self.validate(r.headers['content-type'], result, message_data)


class RequestsBackend(Backend):
    def __init__(self):
        # Lazily import to avoid dependency
        import requests

        self.session = requests.Session()

    def send(self, url, message_data, **kwargs):
        r = self.session.post(url, data=message_data)

        return self.validate(r.headers['Content-Type'], r.text, message_data)


class ConsoleBackend(Backend):
    def send(self, url, message_data, **kwargs):
        print("I: Slack message:")
        pprint.pprint(message_data, indent=4)
        print("-" * 79)


class LoggerBackend(Backend):
    def send(self, url, message_data, **kwargs):
        logger.info(pprint.pformat(message_data, indent=4))


class DisabledBackend(Backend):
    def send(self, url, message_data, **kwargs):
        pass


class CeleryBackend(Backend):
    def __init__(self):
        # Lazily import to avoid dependency
        from .tasks import send

        self._send = send

        # Check we can import our specified backend up-front
        import_string(app_settings.BACKEND_FOR_QUEUE)()

    def send(self, *args, **kwargs):
        # Send asynchronously via Celery
        self._send.delay(*args, **kwargs)


class DjangoQBackend(Backend):
    def __init__(self):
        # Check we can import our specified backend up-front
        import_string(app_settings.BACKEND_FOR_QUEUE)()

    @staticmethod
    def _send(*args, **kwargs):
        backend = import_string(app_settings.BACKEND_FOR_QUEUE)()
        return backend.send(*args, **kwargs)

    def send(self, *args, **kwargs):
        # Send asynchronously via Django-Q
        from django_q.tasks import async_task

        async_task(self._send, *args, group='django-slack', q_options=kwargs)


class TestBackend(Backend):
    """
    This backend is for testing.

    Before a test, call `reset_messages`, and after a test, call
    `retrieve_messages` for a list of all messages that have been sent during
    the test.
    """

    def __init__(self, *args, **kwargs):
        super(TestBackend, self).__init__(*args, **kwargs)
        self.reset_messages()

    def send(self, url, message_data, **kwargs):
        self.messages.append(message_data)

    def reset_messages(self):
        self.messages = []

    def retrieve_messages(self):
        messages = self.messages
        self.reset_messages()
        return messages


# For backwards-compatibility
Urllib2Backend = UrllibBackend
